// -*-C++-*-
/*!
 * @file  ObjectPoseEstimation2ServiceSVC_impl.h
 * @brief Service implementation header of ObjectPoseEstimation2Service.idl
 *
 */

#include "BasicDataTypeSkel.h"
#include "ExtendedDataTypesSkel.h"

#include "ObjectPoseEstimation2ServiceSkel.h"

#include "SRSensorDataStub.h"

#include <stdio.h>

#include "opencv/cvutil/labeler.h"

#include <opencv/cv.h>
#include <opencv/highgui.h>

#include <rtm/CorbaPort.h>
#include "ModelDefinitionCommonServiceStub.h"
#include "EstimationResultViewerServiceStub.h"

#ifndef OBJECTPOSEESTIMATION2SERVICESVC_IMPL_H
#define OBJECTPOSEESTIMATION2SERVICESVC_IMPL_H
 
#define BOX_NUM 1

/*!
 * @class OPEServiceSVC_impl
 * Example class implementing IDL interface ObjectPoseEstimationRTC::idl::OPEService
 */
class OPEServiceSVC_impl
 : public virtual POA_ObjectPoseEstimationRTC::idl::OPEService,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~OPEServiceSVC_impl();

	 ObjectPoseEstimationRTC::idl::Pose3DList ReturnPose3DList;

	 ObjectPoseEstimationRTC::idl::Pose3DList Pose3DListList[3];

	 //fobOp
	 double x[25344];
	 double y[25344];
	 double z[25344];

	RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> *m_MDService;
	RTC::CorbaConsumer<ObjectPoseEstimationRTC::idl::ERVService> *m_ERVService;

	//C[t[`p
	ObjectPoseEstimationRTC::idl::ERVService::Point3DList points;

	int box_count, cylinder_count;

 public:

	 typedef struct _Point3DList
	 {
		 RTC::TimedPoint3D data[25534];
	 } Point3DList;

	 Point3DList** pPoint3DList;
	 SensorRTC::SRSensor::idl::TimedSwissRanger** pSwissRanger;

  /*!
   * @brief standard constructor
   */
	 OPEServiceSVC_impl(Point3DList** pDPPoint3DList, SensorRTC::SRSensor::idl::TimedSwissRanger** pDPSwissRanger, CRITICAL_SECTION* g_CritSec);
	 void setMDInterface(  RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> *MDService)
	 {
		m_MDService = MDService;
	 }

	 void setERVInterface( RTC::CorbaConsumer<ObjectPoseEstimationRTC::idl::ERVService> *ERVService )
	 {
		 m_ERVService = ERVService;
	 }
  /*!
   * @brief destructor
   */
   virtual ~OPEServiceSVC_impl();

   // attributes and operations
   CORBA::Boolean calibrationSensor();
   CORBA::Boolean getBackground();
   ObjectPoseEstimationRTC::idl::Pose3DList *getObjectPose(CORBA::Long id);

   void DrawAmplitudeImg( IplImage **amplitudeimg);
   void DrawDistanceImg( IplImage **distanceimg);
   void DrawConfidenceImg( IplImage **confidenceimg);
   double calcAverage(const std::vector<double>& ori);

};



#endif // OBJECTPOSEESTIMATION2SERVICESVC_IMPL_H


