/*************************************************************************/
/*                                                                       */
/*   icp3d.h: Header file for ICP (Iterative Closest Point)              */
/*                                                                       */
/*************************************************************************/

/*      Copyright(c) 2010, 2011
Jun Miura
		Department of Computer Science and Engineering,
		Toyohashi University of Technology,
		Toyohashi, Aichi 441-8580, Japan
		(TEL) 0532-44-6773  (FAX) 0532-44-6757
		(E-mail) jun.miura@tut.jp                     */

#ifndef ICP3D_H
#define ICP3D_H

#include <opencv/cv.h>

/***********************************************************/
/*                       ^`                            */
/***********************************************************/
/*  2D_  */
typedef struct _point2D
{
  double x, y;
} point2D;

/*  2D  */
typedef struct _line2D
{
  point2D sp, ep;
} line2D;

/*  2Dp`  */
typedef struct _polygon2D
{
  int	line_num;
  line2D	*lines;
} polygon2D;

/*  _  */
typedef struct _point
{
  double x, y, z;
  double nx, ny, nz;	/* @Df[^_݂̂ɗL */
} point;

/*    */
typedef struct _line 
{
  point sp, ep;
} line;

/*  p`  */
typedef struct _polygon
{
  int	line_num;
  line	*lines;
  double nx, ny, nz;	  /* ʂ̖@D̓xNgŒ`ׂ */
} polygon;

/*@3Dʑ́@*/
typedef struct _polytope
{
  int		polygon_num;
  polygon	*polygons;
} polytope;

/*@ʁC*/
typedef struct _disk
{
  point center;
  double radius;
  double nx, ny, nz;
} disk;
/*@~@*/
typedef struct _cylinder
{
  disk disks[2];
} cylinder;

/*  Wϊp[^Fsړ x, y, zƉ̉] tx, ty, rz  */
typedef struct _parameter
{
  double x, y, z, tx, ty, tz;
} parameter;

/***********************************************************/
/*                       萔錾                          */
/***********************************************************/
#define POLYGON_NUM 	6	//ʑ̖̂ʂ̐íj
#define LINE_NUM	4	//ʂ̕ӂ̐íj

#define PP_NUM		13		//6Rx*2 + p[^
#define MAX_DATA_NUM 1000
#define DATA_DIM 3  //f[^_Q̎
#define PCA_DIM	 3  //听͂̎


/***********************************************************/
/*                       ֐錾                          */
/***********************************************************/
/*  define in show-superimpose.c  */
extern void initialize_superimpose();
extern void set_init_vertices(point points[8]);
extern void set_init_cylinder_vertices(point upper[8], point lower[8]);
extern void show_vertices(point vertices[8]);
extern void show_cylinder_vertices(point upper[8], point lower[8]);
extern void show_correspondence(int to_show, point p1, point p2);

/* defined in geo.c */
extern polytope *make_polytope(double wx, double wy, double wz);
extern polytope *new_polytope(int pnum, int lnum);
extern cylinder *make_cylinder(double height, double radius);
extern cylinder *new_cylinder();
extern void copy_polytope(polytope *src, polytope *dst);
extern void transform_polytope(polytope *pp, double dx, double dy, double dz, double tx, double ty, double tz);

extern void copy_cylinder(cylinder *src, cylinder *dst);
extern void transform_cylinder(cylinder *cy, double dx, double dy, double dz, double tx, double ty, double tz);
extern void transform_cylinder_image(point *upper, point *lower, double dx, double dy, double dz, double tx, double ty, double tz);

extern int similar_direction(double nx1, double ny1, double nz1,
		      double nx2, double ny2, double nz2);

extern int similar_direction_side(double nx1, double ny1,double nz1,
						double nx2, double ny2, double nz2);

extern double polygon_point_distance(polygon *polyp, point *pp);
extern double polygon_point_distance_for_find_object(double *average, double *normal, point *pp);
double polygon_point_distance_with_correspondence(polygon *polyp, point *pp, point *cp);

extern double disk_point_distance(disk *diskp, point *pp);
extern double disk_point_distance_with_correspondence(disk *diskp, point *pp, point *cp);

extern double side_point_distance(cylinder *cyp, point *pp);
extern double side_point_distance_with_correspondence(cylinder *cyp, point *pp, point *cp);

extern void convert_point_data(int *point_num, point *points);
void R2EulerXYZ(CvMat *R, double *tx, double *ty, double *tz);
void EulerXYZ2R(double *tx, double *ty, double *tz, CvMat *R);

/*  defined in this file. */
extern void output_scene_positions_to_image(int is_initial,
				     double x, double y, double z,
				     double tx, double ty, double tz);

//~p
extern void output_scene_positions_to_image_cylinder(int is_initial,
				     double x, double y, double z,
				     double tx, double ty, double tz);

extern double calcProjectionY(double *x, double *z, double *normal, CvPoint3D32f average);

extern void calcSurfacePCA(CvPoint3D32f *averagep, CvPoint3D32f *vec1, CvPoint3D32f *vec2, CvPoint3D32f *vec3, 
					double *xval, double *yval, double *zval, int pointnum);

extern double polygon_point_distance_with_correspondence_initialpose(double *normal, CvPoint3D32f average, point *pp, point *cp);


extern parameter object_pose;

//xNgZ
extern void sub_vector( point p1, point p2, point *ans);
extern double distance(point p1, point p2);

extern int check_inside_model(point *pp, cylinder *model);

#endif